import React, { useState, useContext, useEffect } from 'react'
import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import { AppContext } from 'components/Contexts/AppContextProvider'

import StyledButton from 'components/CommonKit/Button/StyledButton'
import StyledBlackSpinner from 'components/CommonKit/Spinner/StyledBlackSpinner'

import { Account, Trigger } from 'doctypes'
import { TriggerService } from 'services/triggersService'
import { AccountService } from 'services/accountService'

import { isKonnectorRunning } from 'cozy-harvest-lib/dist/helpers/triggers'
import {
  KonnectorJob,
  ERROR_EVENT,
  LOGIN_SUCCESS_EVENT,
  SUCCESS_EVENT,
} from 'cozy-harvest-lib/dist/models/KonnectorJob'

interface KonnectorResultProps {
  account: Account
  handleJobState: Function
  client: Client
  t: Function
}

const KonnectorResult: React.FC<KonnectorResultProps> = ({
  account,
  handleJobState,
  client,
  t,
}: KonnectorResultProps) => {
  const [trigger, setTrigger] = useState<Trigger | null>(null)
  const [updating, setUpdating] = useState<boolean>(false)
  const [lastExecutionDate, setLastExecutionDate] = useState<string>('-')
  const [status, setStatus] = useState<string>('')

  const context = useContext(AppContext)

  const updateState = async (trigger: Trigger) => {
    const triggerState = await TriggerService.fetchTriggerState(client, trigger)
    if (triggerState) {
      setLastExecutionDate(
        new Date(triggerState.last_execution).toLocaleString()
      )
      setStatus(triggerState.status)
      handleJobState(triggerState.status)
      await context.refreshFluidTypes()
    }
  }

  const callbackResponse = async () => {
    if (trigger) {
      await updateState(trigger)
    }
    setUpdating(false)
  }

  const updateKonnector = async () => {
    setUpdating(true)
    setStatus('')
    setLastExecutionDate('-')
    handleJobState('')

    if (trigger && !isKonnectorRunning(trigger)) {
      const konnectorJob = new KonnectorJob(client, trigger)
      await konnectorJob.launch()
      konnectorJob.jobWatcher.on(ERROR_EVENT, () => {
        callbackResponse()
      })
      konnectorJob.jobWatcher.on(LOGIN_SUCCESS_EVENT, () => {
        callbackResponse()
      })
      konnectorJob.jobWatcher.on(SUCCESS_EVENT, () => {
        callbackResponse()
      })
    }
  }

  const deleteAccount = async () => {
    setUpdating(true)
    try {
      if (account) {
        await AccountService.deleteAccount(client, account)
        await context.refreshFluidTypes()
      }
    } catch (error) {
      setUpdating(false)
    }
  }

  useEffect(() => {
    let subscribed = true
    async function getData() {
      const _trigger = await TriggerService.fetchTriggerFromAccount(
        client,
        account
      )
      if (subscribed && _trigger) {
        setTrigger(_trigger)
        await updateState(_trigger)
      }
    }
    getData()
    return () => {
      subscribed = false
    }
  }, [])

  return (
    <div className="accordion-update-result">
      <div className="accordion-update">
        <div
          className={
            status === 'errored'
              ? 'accordion-caption-red text-16-normal'
              : 'accordion-caption text-16-normal'
          }
        >
          {t('KONNECTORCONFIG.LABEL_UPDATEDAT')}
        </div>
        <div>{lastExecutionDate}</div>
      </div>
      <div className="inline-buttons">
        <StyledButton
          type="button"
          color="primary"
          onClick={updateKonnector}
          disabled={updating}
        >
          {updating ? (
            <StyledBlackSpinner size="2em" />
          ) : (
            <div>{t('KONNECTORCONFIG.BTN_UPDATE')}</div>
          )}
        </StyledButton>
        <StyledButton
          type="button"
          color="secondary"
          onClick={deleteAccount}
          disabled={updating}
        >
          {t('KONNECTORCONFIG.BTN_DELETE')}
        </StyledButton>
      </div>
    </div>
  )
}

export default translate()(withClient(KonnectorResult))
